/**
 * @author Olgierd Humeńczuk olgierdh@gmail.com
 * @file main
 */

#include <iostream>
#include <stdlib.h>
#include <stdexcept>
#include <vector>

#ifdef WIN32
#define WIN32_LEAN_AND_MEAN
#include <windows.h>
#include <SDL.h>
#else
#include <SDL/SDL.h>
#endif

#include <GL/gl.h>
#include <GL/glu.h>

#undef main

/*static const GLfloat cubeArray[][3] =
{
    {-0.5f, -0.5f, -0.5f}, {-0.5f, 0.5f, -0.5f}, {0.5f, 0.5f, -0.5f}, {0.5f, -0.5f, -0.5f},
    {-0.5f, -0.5f, 0.5f}, {0.5f, -0.5f, 0.5f}, {0.5f, 0.5f, 0.5f}, {-0.5f, 0.5f, 0.5f},
    {-0.5f, -0.5f, -0.5f}, {0.5f, -0.5f, -0.5f}, {0.5f, -0.5f, 0.5f}, {-0.5f, -0.5f, 0.5f},
    {-0.5f, 0.5f, -0.5f}, {-0.5f, 0.5f, 0.5f}, {0.5f, 0.5f, 0.5f}, {0.5f, 0.5f, -0.5f},
    {-0.5f, 0.5f, -0.5f}, {-0.5f, -0.5f, -0.5f}, {-0.5f, -0.5f, 0.5f}, {-0.5f, 0.5f, 0.5f},
    {0.5f, -0.5f, -0.5f}, {0.5f, 0.5f, -0.5f}, {0.5f, 0.5f, 0.5f}, {0.5f, -0.5f, 0.5f}
};

//deklaracja wierzchołków płaszczyzny
static const GLfloat planeArray[][3] =
{
    { -0.5f, -0.5f, -0.5f }, { -0.5f, -0.5f, 0.5f }, { 0.5f, -0.5f, 0.5f }, { 0.5f, -0.5f, -0.5f }
};*/


/**
 * @brief Funkcja przetwarza zdarzenia pochodzace z kontrolerów takich jak:
 *          * klawiatura
 *          * mysz komputerowa
 *
 * @return true jeżeli aplikacja ma pozostać otwarta, false wpp
 */
bool processInput( void )
{
    // zmienna pomocnicza
    SDL_Event event;

    // czy są jakieś czekające na przetworzenie zdarzenia ?
    if ( SDL_PollEvent( &event ) )
    {
        // sprawdzenie typu zdarzenia
        switch( event.type )
        {
            // wyjście z aplikacji
            case SDL_QUIT:
                return false;

            // użytkownik nacisnął któryś z klawiszy na klawiaturze
            case SDL_KEYDOWN:
                // jeżeli nacisnał klawisz eskape zamykamy aplikacje
                if( event.key.keysym.sym == SDLK_ESCAPE )
                {
                    return false;
                }
                break;
            case SDL_MOUSEMOTION:
                break;
        }
    }
    return true;
}
/**
 * @brief Funkcja odpowiadająca za inicjalizację biblioteki SDL
 *          oraz utworzenie okna z kontekstem OpenGL.
 * @param scrWidth szerokosc ekranu
 * @param scrHeight wysokosc ekranu
 *
 * @throw std::runtime_error jeżeli nastąpi jakikolwiek błąd.
 */
void initSDL( int scrWidth, int scrHeight )
{
    // zaincjalizowanie części biblioteki odpowiadającej za wyświetlanie
    if( SDL_Init( SDL_INIT_VIDEO ) < 0 )
    {
        throw std::runtime_error( "Can't initialize SDL\n" );
    }

    // ustawienie ilości bitów dla każdego z kolorów
    SDL_GL_SetAttribute( SDL_GL_RED_SIZE, 8 );
    SDL_GL_SetAttribute( SDL_GL_GREEN_SIZE, 8 );
    SDL_GL_SetAttribute( SDL_GL_BLUE_SIZE, 8 );

    // ustawienie ilości bitów dla specjalnych buforów
    SDL_GL_SetAttribute( SDL_GL_DEPTH_SIZE, 24 );
    SDL_GL_SetAttribute( SDL_GL_STENCIL_SIZE, 8 );

    // włączenie podwójnego buforowania
    SDL_GL_SetAttribute( SDL_GL_DOUBLEBUFFER, true );
    SDL_GL_SetAttribute( SDL_GL_SWAP_CONTROL, 1 );

    // stworzenie okna aplikacji z kontekstem OpenGL'a
    if ( SDL_SetVideoMode( scrWidth, scrHeight, 24, SDL_OPENGL ) == 0 )
    {
        throw std::runtime_error(
            std::string( "Unable to set video mode: " ).append(
            	std::string( SDL_GetError() ) ) );
    }
}

/**
 * @brief Iniclalizacja stanu poczatkowego niektorych
 *         wartosci OpenGL.
 * @param width szerokosc ekranu
 * @param height wysokosc ekranu
 */
void initOpenGL( int width, int height )
{
    glMatrixMode( GL_PROJECTION );
    gluPerspective( 45.0f, width / ( GLfloat ) height, 0.1f, 100.0f );

    glMatrixMode( GL_MODELVIEW );
    glLoadIdentity();

    glPolygonMode( GL_FRONT_AND_BACK, GL_LINE );
}

/**
 * @struct vertex3f
 * @brief Struktura opisujaca wierzchołek w przestrzeni 3d
 */
struct vertex3f
{
    /**
     * @brief Domyslny konstruktor
     * Ustawia wszystkie wspolrzedne na ( 0, 0, 0 )
     */
    vertex3f( void ) : x( .0f ), y( .0f ), z( .0f )
    {
    }

    /**
     * @brief Konstruktor ustawiajacy wspolrzedne
     * @param in_x wspolrzedna x
     * @param in_y wspolrzedna y
     * @param in_z wspolrzedna z
     */
    vertex3f( GLfloat in_x, GLfloat in_y, GLfloat in_z )
        : x( in_x ), y( in_y ), z( in_z )
    {
    }

    ///wspolrzedna x
    GLfloat x;
    ///wspolrzedna y
    GLfloat y;
    ///wspolrzedna z
    GLfloat z;
};

/** Deklaracja typu tablicy wierzcholkow */
typedef std::vector< vertex3f > vertexes;

/**
 * @brief Tworzy tablice wierzcholkow szescianu
 * @param width dlugosc boku szescianu
 */
vertexes createCube( GLfloat width )
{
    vertexes ret;
    //rezerwacja pamięci
    ret.reserve( 6 * 4 * sizeof( vertexes::value_type ) );

    //zapamiętanie wartości pomocniczych
    GLfloat min = -width * 0.5f;
    GLfloat max = -min;

    //przednia sciana
    ret.push_back( vertex3f( min, max, max ) );
    ret.push_back( vertex3f( min, min, max ) );
    ret.push_back( vertex3f( max, min, max ) );
    ret.push_back( vertex3f( max, max, max ) );

    //tylnia sciana
    ret.push_back( vertex3f( max, max, min ) );
    ret.push_back( vertex3f( max, min, min ) );
    ret.push_back( vertex3f( min, min, min ) );
    ret.push_back( vertex3f( min, max, min ) );

    //prawa sciana
    ret.push_back( vertex3f( max, max, max ) );
    ret.push_back( vertex3f( max, min, max ) );
    ret.push_back( vertex3f( max, min, min ) );
    ret.push_back( vertex3f( max, max, min ) );

    //lewa sciana
    ret.push_back( vertex3f( min, max, min ) );
    ret.push_back( vertex3f( min, min, min ) );
    ret.push_back( vertex3f( min, min, max ) );
    ret.push_back( vertex3f( min, max, max ) );

    //gorna sciana
    ret.push_back( vertex3f( min, max, min ) );
    ret.push_back( vertex3f( min, max, max ) );
    ret.push_back( vertex3f( max, max, max ) );
    ret.push_back( vertex3f( max, max, min ) );

    //dolna sciana
    ret.push_back( vertex3f( min, min, max ) );
    ret.push_back( vertex3f( min, min, min ) );
    ret.push_back( vertex3f( max, min, min ) );
    ret.push_back( vertex3f( max, min, max ) );

    return ret;
}

/**
 * @brief Główna metoda programu ( entry point )
 * @param argc ilość argumentów przekazanych do programu
 * @param argv tablica wartości argumentów
 */
int main( const int /*argc*/, const char** /*argv*/ )
{
	 const int screenX = 800;
	 const int screenY = 600;

    try
    {
        initSDL( screenX, screenY );
        initOpenGL( screenX, screenY );

        vertexes cubeVertexes = createCube( 2.0f );

        glEnableClientState( GL_VERTEX_ARRAY) ;

        // główna pętla
        while( processInput() )
        {
            glClear( GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT );
            glMatrixMode( GL_MODELVIEW );
            glLoadIdentity();

            glPushMatrix();
				gluLookAt( 0.0f, 0.0f, 10.0f, 0.0f, 0.0f, 0.0f, 0.0f, 1.0f, 0.0f );
            glVertexPointer( 3, GL_FLOAT, 0, &cubeVertexes[ 0 ] );
            glDrawArrays( GL_QUADS, 0, cubeVertexes.size() );
            glPopMatrix();


            SDL_GL_SwapBuffers();
        }
    }
    catch( const std::exception& e )
    {
        std::cerr << "Exception caught: " << e.what() << std::endl;
    }

    SDL_Quit();
    return 0;
}
